package clustering.poboc;

import processutilities.core.OutputHandler;
import report.Report;
import similarity.SimilarityFunction;
import clustering.interfaces.ClusteringParam;
import clustering.interfaces.ClusteringParametersContainer;
import clustering.interfaces.DatabaseParam;
import clustering.interfaces.ProgressMonitor;
import clustering.interfaces.SimilarityFunctionParam;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.interfaces.DataSet;
import datastructures.interfaces.Row;

public class PoBOCParametersContainer implements ClusteringParametersContainer {

	private DataSet dataSet;
	private OutputHandler<Row> outputHandler;
	private ProgressMonitor progressMonitor;
	private SimilarityFunction similarityFunction;
	private DatabaseConnectionProvider database;	
	private double samplingFactor = 1.0f;
	private Report report;
	
	@Override
	public DataSet getDataSet() {
		return dataSet;
	}
	
	@Override
	public void setDataSet(DataSet dataSet) {
		this.dataSet = dataSet;
	}

	@Override
	public OutputHandler<Row> getOutputHandler() {
		return outputHandler;
	}

	@Override
	public void setOutputHandler(OutputHandler<Row> outputHandler) {
		this.outputHandler = outputHandler;
	}
	
	@Override
	public ProgressMonitor getProgressMonitor() {
		return progressMonitor;
	}

	@Override
	public void setProgressMonitor(ProgressMonitor progressMonitor) {
		this.progressMonitor = progressMonitor;
	}
	
	@SimilarityFunctionParam
	public SimilarityFunction getSimilarityFunction() {
		return similarityFunction;
	}
	
	public void setSimilarityFunction(SimilarityFunction similarityFunction) {
		this.similarityFunction = similarityFunction;
	}
	
	@DatabaseParam
	public DatabaseConnectionProvider getDataBase() {
		return database;
	}
	
	public void setDataBase(DatabaseConnectionProvider database) {
		this.database = database;
	}
	
	@ClusteringParam(name = "Sampling factor", 
			description = "The dataset will be sampled by this factor " +
					"when building the similarity matrix, " +
					"so its size will be SF * N^2. " +
					"It must be between 0 and 1.", mandatory = false)
	public double getSamplingFactor() {
		return samplingFactor;
	}
	
	public void setSamplingFactor(double samplingFactor) {
		if(samplingFactor <= 0.0 || samplingFactor > 1.0) {
			throw new IllegalArgumentException("The sampling factor must be between 0 and 1.");
		}
		this.samplingFactor = samplingFactor;
	}

	@Override
	public Report getReport() {
		if(report == null)
			report = new Report();
		return report;
	}

	@Override
	public void setReport(Report report) {
		this.report = report;	
	}
}
