package clustering.poboc;

import static org.junit.Assert.*;

import java.sql.SQLException;

import org.junit.Before;
import org.junit.Test;

import report.Report;
import similarity.SimilarityFunction;
import similarity.lpnorms.EuclideanSimilarity;

import clustering.helpers.TextProgressMonitor;
import clustering.interfaces.ProgressMonitor;
import clustering.poboc.sql.ClustersTable;
import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.extra.MemoryDataSet;
import datastructures.io.DataSetChecker;
import datastructures.simple.SimpleNominalAttribute;
import datastructures.simple.SimpleNumericAttribute;
import datastructures.simple.SimpleRelation;

public class PoBOCAlgorithmTest {
	
	private PoBOCAlgorithm instance;
	
	private SimilarityFunction simFunction;
	
	private MemoryDataSet mockDataSet;
	
	private SimpleRelation relation;

	private DatabaseConnectionProvider database;
	
	private ProgressMonitor progressMonitor;
	
	private static String[][] csv = new String[][] { 
		{ "4", "3" }, // 0
		{ "2", "5" }, // 1
		{ "5", "6" }, // 2
		{ "14", "9" }, // 3
		{ "13", "11" }, // 4
		{ "18", "10" }, // 5
		{ "10.5", "8" }, // 6
		{ "10", "4" } // 7
	};
	
	private static String[][] csvOut = new String[][] { 
		{ "4", "3","F","T" }, // 0
		{ "2", "5","F","T" }, // 1
		{ "5", "6","F","T" }, // 2
		{ "14", "9","T","F" }, // 3
		{ "13", "11","T","F" }, // 4
		{ "18", "10","T","F" }, // 5
		{ "10.5", "8","T","F" }, // 6
		{ "10", "4","F","T" } // 7
	};

	@Before
	public void setUp() throws Exception {
		relation = new SimpleRelation("PoBOCAlgorithmTest");
		relation.addAttribute(new SimpleNumericAttribute("first"));
		relation.addAttribute(new SimpleNumericAttribute("second"));
		mockDataSet = new MemoryDataSet(csv, relation);
		// dataBase = new DerbyConnectionProvider("PoBOCAlgorithmTest");
		database = new HSQLDBConnectionProvider("PoBOCAlgorithmTest");
		simFunction = new EuclideanSimilarity();
		progressMonitor = new TextProgressMonitor();
		
		instance = new PoBOCAlgorithm();
		new Report();
	}

	@Test
	public void testBuildClusters() throws SQLException {
		ClustersTable ct = instance.buildClusters(mockDataSet, database, 1.0, simFunction, progressMonitor);
		assertNotNull(ct);
		
		//instance.getSimilaritiesTable().printTable();
//		instance.getAveragesTable().printTable();
//		instance.getPolesTable().printTable();
//		instance.getConnectionsTable().printTable();
		instance.getClustersTable().printTable();
		
		assertEquals(2, ct.getClusters().length);
	}

	@Test
	public void testDoClustering() {
		SimpleRelation generatedRelation = new SimpleRelation("LinkSysAlgorithmTest");
		generatedRelation.addAttribute(new SimpleNumericAttribute("first"));
		generatedRelation.addAttribute(new SimpleNumericAttribute("second"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("c0","T","F"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("c1","T","F"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("c2","T","F"));
		
		DataSetChecker checker = new DataSetChecker(csvOut,generatedRelation,true);
		
		PoBOCParametersContainer pc = new PoBOCParametersContainer();
		pc.setDataSet(mockDataSet);
		pc.setProgressMonitor(progressMonitor);
		pc.setSimilarityFunction(simFunction);
		pc.setOutputHandler(checker);
		pc.setDataBase(database);
		pc.setReport(new Report());
		
		instance.doClustering(pc);
		
		assertTrue(checker.isRunned());
	}
}
