package clustering.poboc;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import similarity.matrix.SimilaritiesTable;

import clustering.poboc.sql.ClustersTable;
import clustering.poboc.sql.PolesTable;
import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;

public class PoBOCResultFetcherTest {

	private PoBOCResultFetcher instance;
	private DatabaseConnectionProvider database;
	private ClustersTable clustersTable;
	
	@Before
	public void setUp() throws Exception {
		database = new HSQLDBConnectionProvider("PoBOCResultFetcherTest");
		(new SimilaritiesTable(database.connect())).close();
		PolesTable polesTable = new PolesTable(database.connect());
		clustersTable = new ClustersTable(database.connect(),polesTable);
		
		clustersTable.insertCluster(1, 0);
		clustersTable.insertCluster(1, 1);
		clustersTable.insertCluster(2, 0);
		clustersTable.insertCluster(3, 1);
		clustersTable.insertCluster(4, 2);
		clustersTable.insertCluster(5, 2);
		
		
		instance = new PoBOCResultFetcher(clustersTable);
	}

	@Test
	public void testGetClusters() {
		assertArrayEquals(new String[]{"c0","c1","c2"},instance.getClusters());
	}

	@Test
	public void testGetClustersArray() {
		assertBooleansEquals(new boolean[]{true,true,false}, instance.getClustersArray(1));
		assertBooleansEquals(new boolean[]{true,false,false}, instance.getClustersArray(2));
		assertBooleansEquals(new boolean[]{false,true,false}, instance.getClustersArray(3));
		assertBooleansEquals(new boolean[]{false,false,true}, instance.getClustersArray(4));
		assertBooleansEquals(new boolean[]{false,false,true}, instance.getClustersArray(5));
	}
	
	private void assertBooleansEquals(boolean[] b1, boolean[] b2) {
		assertNotNull(b2);
		assertEquals(b1.length,b2.length);
//		System.out.println("AssertArrayEquals!!");
//		for(int i=0; i < b1.length; i++) {
//			System.out.println(b2[i]);
//		}
		for(int i=0; i < b1.length; i++) {
			assertEquals("i = " + i,b1[i],b2[i]);
		}
	}
}
