package clustering.poboc;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

import similarity.lpnorms.EuclideanSimilarity;

import clustering.helpers.TextProgressMonitor;

import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.extra.MemoryDataSet;
import datastructures.io.DataSetChecker;
import datastructures.simple.SimpleNominalAttribute;
import datastructures.simple.SimpleNumericAttribute;
import datastructures.simple.SimpleRelation;

public class PoBOCSingleLinkAlgorithmTest {

	private DatabaseConnectionProvider db;
	private SimpleRelation relation;
	private String[][] csvIn = {
			{"1","1"},
			{"2","2"},
			{"5","5"},
			{"8","8"},
			{"9","9"},
			{"1","8"},
			{"2","7"}
	};
	private String[][] csvOut = {
			{"1","1","F","T","F","T"},
			{"2","2","F","T","F","T"},
			{"5","5","F","F","T","T"},
			{"8","8","T","F","F","F"},
			{"9","9","T","F","F","F"},
			{"1","8","F","F","T","T"},
			{"2","7","F","F","T","T"}
	};
	private MemoryDataSet mockDataSet;
	private PoBOCParametersContainer params;
	
	@Before
	public void setUp() throws Exception {
		db = new HSQLDBConnectionProvider("PoBOC");

		relation = new SimpleRelation("PoBOCAlgorithmTest");
		relation.addAttribute(new SimpleNumericAttribute("first"));
		relation.addAttribute(new SimpleNumericAttribute("second"));
		
		mockDataSet = new MemoryDataSet(csvIn , relation);
		params = new PoBOCParametersContainer();
		params.setDataBase(db);
		params.setDataSet(mockDataSet);
		params.setProgressMonitor(new TextProgressMonitor());
		params.setSimilarityFunction(new EuclideanSimilarity());
	}

	@Test
	public void testDoClustering() {
		SimpleRelation generatedRelation = new SimpleRelation("LinkSysAlgorithmTest");
		generatedRelation.addAttribute(new SimpleNumericAttribute("first"));
		generatedRelation.addAttribute(new SimpleNumericAttribute("second"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("c0","T","F"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("c1","T","F"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("c2","T","F"));
		generatedRelation.addAttribute(new SimpleNominalAttribute("g3","T","F"));
		
		
		DataSetChecker checker = new DataSetChecker(csvOut,generatedRelation,true);
		
		params.setOutputHandler(checker);
		
		(new PoBOCSingleLinkAlgorithm()).doClustering(params);
		
		assertTrue(checker.isRunned());
		
		//System.out.println(params.getReport().toString());
	}

}
