package clustering.poboc.sql;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import similarity.SimilarityFunction;
import similarity.lpnorms.EuclideanSimilarity;
import similarity.matrix.MatrixFiller;
//import database.core.DerbyConnectionProvider;
import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.extra.MemoryDataSet;
import datastructures.simple.SimpleNumericAttribute;
import datastructures.simple.SimpleRelation;

public class CliqueConstructorTest {

	private CliqueConstructor cliqueConstructor;
	private MatrixFiller matrixFiller;
	private DatabaseConnectionProvider dataBase;
	SimilarityFunction simFunction;
	private MemoryDataSet mockDataSet;
	private static String[][] csv = new String[][] { { "4", "3" }, // 0
			{ "2", "5" }, // 1
			{ "5", "6" }, // 2
			{ "14", "9" }, // 3
			{ "13", "11" }, // 4
			{ "18", "10" }, // 5
	};
	private SimpleRelation originalRelation;
	
	@Before
	public void setUp() throws Exception {
		originalRelation = new SimpleRelation("Original");
		originalRelation.addAttribute(new SimpleNumericAttribute("first"));
		originalRelation.addAttribute(new SimpleNumericAttribute("second"));
		mockDataSet = new MemoryDataSet(csv, originalRelation);
		//dataBase = new DerbyConnectionProvider("DatabaseTestMatrixFiller");
		dataBase = new HSQLDBConnectionProvider("DatabaseTestMatrixFiller");
		simFunction = new EuclideanSimilarity();
		matrixFiller = new MatrixFiller(true, mockDataSet, dataBase,simFunction);
		matrixFiller.fillMatrix();
	}

	@After
	public void tearDown() throws Exception {
		dataBase.close();
	}

	@Test
	public void testBuildCliques() throws SQLException {
		
		this.cliqueConstructor= new CliqueConstructor(this.dataBase.connect(), this.matrixFiller.getSimilaritiesTable(), this.matrixFiller.getAveragesTable());
		cliqueConstructor.buildCliques(matrixFiller.getAveragesTable().getTotalAverage());
		
		PolesTable poleTable = cliqueConstructor.getPoleTable();
				
		ArrayList<Integer> arrayObjects = new ArrayList<Integer>();

		// object connected expected: 3,4,5
		ResultSet rs = poleTable.getPoleObjects(0);
		while (rs.next()) {
			arrayObjects.add(rs.getInt(1));
		}

		assertEquals(3, arrayObjects.size());
		assertTrue(arrayObjects.contains(3));
		assertTrue(arrayObjects.contains(4));
		assertTrue(arrayObjects.contains(5));
		
		arrayObjects.clear();
		
		// object connected expected: 0,1,2
		rs = poleTable.getPoleObjects(1);
		while (rs.next()) {
			arrayObjects.add(rs.getInt(1));
		}

		assertEquals(3, arrayObjects.size());
		assertTrue(arrayObjects.contains(0));
		assertTrue(arrayObjects.contains(1));
		assertTrue(arrayObjects.contains(2));
		
		poleTable.printTable();

		
	}


}
