package clustering.poboc.sql;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import similarity.SimilarityFunction;
import similarity.lpnorms.EuclideanSimilarity;
import similarity.matrix.MatrixFiller;
import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.extra.MemoryDataSet;
import datastructures.simple.SimpleNumericAttribute;
import datastructures.simple.SimpleRelation;

public class ClustersTableTest {

	private ClustersTable instance;
	private CliqueConstructor cliqueConstructor;
	private MatrixFiller matrixFiller;
	private DatabaseConnectionProvider dataBase;
	SimilarityFunction simFunction;
	private MemoryDataSet mockDataSet;
	private static String[][] csv = new String[][] { { "4", "3" }, // 0
			{ "2", "5" }, // 1
			{ "5", "6" }, // 2
			{ "14", "9" }, // 3
			{ "13", "11" }, // 4
			{ "18", "10" }, // 5
			{ "10.5" , "8" }, // 6
			{ "10" , "4" } // 7
	};
	private SimpleRelation originalRelation;
	
	@Before
	public void setUp() throws Exception {
		originalRelation = new SimpleRelation("Original");
		originalRelation.addAttribute(new SimpleNumericAttribute("first"));
		originalRelation.addAttribute(new SimpleNumericAttribute("second"));
		mockDataSet = new MemoryDataSet(csv, originalRelation);
		//dataBase = new DerbyConnectionProvider("DatabaseTestMatrixFiller");
		dataBase = new HSQLDBConnectionProvider("DatabaseTestMatrixFiller");
		simFunction = new EuclideanSimilarity();
		matrixFiller = new MatrixFiller(true, mockDataSet, dataBase,simFunction);
		matrixFiller.fillMatrix();
		
		cliqueConstructor= new CliqueConstructor(this.dataBase.connect(), this.matrixFiller.getSimilaritiesTable(), this.matrixFiller.getAveragesTable());
		cliqueConstructor.buildCliques(matrixFiller.getAveragesTable().getTotalAverage());
		
	}

	@After
	public void tearDown() throws Exception {
		try {
			instance.close();
		} catch(Exception e) {
		} finally {
			dataBase.close();
		}		
	}

	@Test
	public void testClustering() throws SQLException {
		
		instance = new ClustersTable(dataBase.connect(), cliqueConstructor.getPoleTable());
		instance.doClustering(this.matrixFiller.getSimilaritiesTable());
		
		cliqueConstructor.getSimTable().printTable();
		cliqueConstructor.getAvgTable().printTable();
		cliqueConstructor.getConnTable().printTable();
		cliqueConstructor.getPoleTable().printTable();		
		instance.printTable();
		
		int id = 1;
		ResultSet rs= cliqueConstructor.getPoleTable().getPoleSimilarities(id);
		while (rs.next())
		{
			System.out.println("Id= "+id+" Pole= "+rs.getInt(1)+" PoleSim= "+rs.getFloat(2));
		}
	}

	@Test
	public void testInsertCluster() throws SQLException {
		
		instance= new ClustersTable(dataBase.connect(),  cliqueConstructor.getPoleTable());
		instance.insertCluster(1, 0);
		instance.insertCluster(1, 1);
		instance.insertCluster(2, 0);
		instance.insertCluster(3, 1);
		instance.insertCluster(4, 2);
		instance.insertCluster(5, 2);
	
		int[] idClusters = instance.getClusters();
		Arrays.sort(idClusters);
		assertArrayEquals(new int[]{0,1,2}, idClusters);
		
		int[] idObjectClusters = instance.getClusters(1);
		Arrays.sort(idObjectClusters);
		assertArrayEquals(new int[]{0,1}, idObjectClusters);		
	}
	
	@Test
	public void testGetSimClusters() throws SQLException
	{
		instance = new ClustersTable(dataBase.connect(), cliqueConstructor.getPoleTable());
		instance.doClustering(this.matrixFiller.getSimilaritiesTable());
		
		int numberCluster = instance.getClusters().length;
		float sim;
		for (int i=0; i<numberCluster; i++)
		{
			for (int j=0; j<numberCluster; j++)
			{
				sim= instance.getSimClusters(i, j);
				System.out.println("C1= "+i+" C2= "+j+" Sim(C1,C2)= "+sim);
			}			
		}
		instance.close();
	}
	
	@Test
	public void testNumber() throws SQLException {
		
		instance= new ClustersTable(dataBase.connect(),  cliqueConstructor.getPoleTable());
		instance.insertCluster(1, 0);
		instance.insertCluster(1, 1);
		instance.insertCluster(2, 0);
		instance.insertCluster(3, 1);
		instance.insertCluster(4, 2);
		instance.insertCluster(5, 2);
	
		instance.printTable();
		
		assertEquals(6, instance.getSize());
		assertEquals(2, instance.getNumberOfElementsInCluster(1));
		assertEquals(3, instance.getNumberOfClustersFound());
	
	}


}
