package clustering.poboc.sql;

import static org.junit.Assert.*;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import similarity.SimilarityFunction;
import similarity.lpnorms.EuclideanSimilarity;
import similarity.matrix.MatrixFiller;
//import database.core.DerbyConnectionProvider;
import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.extra.MemoryDataSet;
import datastructures.simple.SimpleNumericAttribute;
import datastructures.simple.SimpleRelation;

public class ConnectionsTableTest {

	private ConnectionsTable connTable;
	private MatrixFiller matrixFiller;
	private DatabaseConnectionProvider dataBase;
	SimilarityFunction simFunction;
	private MemoryDataSet mockDataSet;
	private static String[][] csv = new String[][] { { "4", "3" }, // 0
			{ "2", "5" }, // 1
			{ "5", "6" }, // 2
			{ "14", "9" }, // 3
			{ "13", "11" }, // 4
			{ "18", "10" }, // 5
	};
	private SimpleRelation originalRelation;

	@Before
	public void setUp() throws Exception {
		originalRelation = new SimpleRelation("Original");
		originalRelation.addAttribute(new SimpleNumericAttribute("first"));
		originalRelation.addAttribute(new SimpleNumericAttribute("second"));
		mockDataSet = new MemoryDataSet(csv, originalRelation);
		//dataBase = new DerbyConnectionProvider("DatabaseTestMatrixFiller");
		dataBase = new HSQLDBConnectionProvider("DatabaseTestMatrixFiller");
		simFunction = new EuclideanSimilarity();
		matrixFiller = new MatrixFiller(true, mockDataSet, dataBase,
				simFunction);
		matrixFiller.fillMatrix();
	}
	
	@After
	public void tearDown() throws Exception {
		dataBase.close();
	}

	@Test
	public void testFillConnectionTable() throws SQLException {

		matrixFiller.getAveragesTable().printTable();
		
		Connection conn = dataBase.connect();
		connTable = new ConnectionsTable(conn);

		try {
			connTable.fillConnectionTable(matrixFiller.getSimilaritiesTable(),
					matrixFiller.getAveragesTable());
		} catch (SQLException e) {
			e.printStackTrace();
		}

		ArrayList<Integer> arrayObjects = new ArrayList<Integer>();

		// object connected expected: 0,1,2
		ResultSet rs = connTable.getConnection(0);
		while (rs.next()) {
			arrayObjects.add(rs.getInt(1));
		}

		assertEquals(2, arrayObjects.size());
		assertTrue(arrayObjects.contains(1));
		assertTrue(arrayObjects.contains(2));

		arrayObjects.clear();
		
		// object connected expected: 3,5
		rs = connTable.getConnection(4);
		while (rs.next()) {
			System.out.println(rs.getInt(1));
			arrayObjects.add(rs.getInt(1));
		}

		assertEquals(2, arrayObjects.size());
		assertTrue(arrayObjects.contains(3));
		assertTrue(arrayObjects.contains(5));
				
		PreparedStatement ps = conn.prepareStatement(
				"SELECT S.Id_i, S.Id_j " +
				"FROM Similarities S JOIN Averages A ON (S.Id_i = A.Id) " +
				//CAGOSA
//				"WHERE S.Id_i = A.Id AND S.Sim >= ALL (" +
//					"SELECT A2.RowAvg " +
//					"FROM Averages A2 " +
//					"WHERE A2.Id = S.Id_i OR A2.Id = S.Id_j )";
				//GIA MEGLIO
				"WHERE S.Sim >= (" +
					"SELECT A2.RowAvg " +
					"FROM Averages A2 " +
					"WHERE A2.Id = S.Id_i ) " +
				"AND S.Sim >= (" +
					"SELECT A2.RowAvg " +
					"FROM Averages A2 " +
					"WHERE A2.Id = S.Id_j )");
		//	"WHERE S.Id_i = A.Id AND S.Sim >= ?";
		//ps.setFloat(1, 0.5f);
		rs = ps.executeQuery();
		while(rs.next()) {
			//System.out.println("pua");
		}
		rs.close();
		ps.close();

		connTable.printTable();
	}

}
