package clustering.poboc.sql;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import similarity.lpnorms.EuclideanSimilarity;
import similarity.matrix.MatrixFiller;
import similarity.matrix.SimilaritiesTable;

// import database.core.DerbyConnectionProvider;
import database.core.HSQLDBConnectionProvider;
import database.interfaces.DatabaseConnectionProvider;
import datastructures.extra.MemoryDataSet;
import datastructures.simple.SimpleNumericAttribute;
import datastructures.simple.SimpleRelation;

public class PolesTableTest {

	private DatabaseConnectionProvider database;
	private PolesTable instance;

	@Before
	public void setUp() throws Exception {
		database=new HSQLDBConnectionProvider("SimilaritiesTableTest");
//		database=new DerbyConnectionProvider("SimilaritiesTableTest");
		
	}

	@After
	public void tearDown() throws Exception {
		try {
			instance.close();
		} catch(Exception e) {
		} finally {
			database.close();
		}
	}

	@Test
	public void testInsertPole() throws SQLException {
		
		(new SimilaritiesTable(database.connect())).close();
		instance= new PolesTable(database.connect());
		instance.insertPole(3, 0);
		instance.insertPole(4, 0);
		instance.insertPole(5, 0);
		instance.printTable();
		
		ArrayList<Integer> arrayObjects = new ArrayList<Integer>();

		// object expected: 3,4,5
		ResultSet rs = instance.getPoleObjects(0);
		while (rs.next()) {
			arrayObjects.add(rs.getInt(1));
		}

		assertEquals(3, arrayObjects.size());
		assertTrue(arrayObjects.contains(3));
		assertTrue(arrayObjects.contains(4));
		assertTrue(arrayObjects.contains(5));
	
	}

	@Test
	public void testGetPoleObjects() throws SQLException {
		
		(new SimilaritiesTable(database.connect())).close();
		instance= new PolesTable(database.connect());
		instance.insertPole(3, 0);
		instance.insertPole(4, 0);
		instance.insertPole(5, 0);
		
		ResultSet rs = instance.getPoleObjects(1);		
		assertFalse(rs.next()); // Pole inesistente
		rs.close();
		
		instance.deletePole(0);
		rs = instance.getPoleObjects(0);		
		assertFalse(rs.next()); // Pole inesistente perchè appena cancellato
	}
	
	@Test
	public void testGetPoleSimilarities() throws SQLException {
		
		String[][] csv = new String[][] { { "4", "3" }, // 0
			{ "2", "5" }, // 1
			{ "5", "6" }, // 2
			{ "14", "9" }, // 3
			{ "13", "11" }, // 4
			{ "18", "10" }, // 5
			{ "7" , "10" }, // 6
			{ "10" , "4" } // 7
		};
	
		SimpleRelation originalRelation = new SimpleRelation("Original");
		originalRelation.addAttribute(new SimpleNumericAttribute("first"));
		originalRelation.addAttribute(new SimpleNumericAttribute("second"));
		MemoryDataSet mockDataSet = new MemoryDataSet(csv, originalRelation);
		EuclideanSimilarity simFunction = new EuclideanSimilarity();
		MatrixFiller matrixFiller = new MatrixFiller(true, mockDataSet, database,simFunction);
		matrixFiller.fillMatrix();
				
		CliqueConstructor cliqueConstructor = new CliqueConstructor(database.connect(), matrixFiller.getSimilaritiesTable(), matrixFiller.getAveragesTable());
		cliqueConstructor.buildCliques(matrixFiller.getAveragesTable().getTotalAverage());
		
		instance = cliqueConstructor.getPoleTable();
		instance.printTable();
		
		int id=6;
		ResultSet rs= instance.getPoleSimilarities(id);
		while (rs.next())
		{
			System.out.println("Id= "+id+" Pole= "+rs.getInt(1)+" PoleSim= "+rs.getFloat(2));
		}
		
		id = 2;
		rs = instance.getPoleObjects(id);
		while (rs.next())
		{
			System.out.println("Id= "+rs.getInt(1)+" Pole= "+id);
		}
	}
	
	@Test
	public void testGetNumber() throws SQLException {
		
		String[][] csv = new String[][] { { "4", "3" }, // 0
			{ "2", "5" }, // 1
			{ "5", "6" }, // 2
			{ "14", "9" }, // 3
			{ "13", "11" }, // 4
			{ "18", "10" }, // 5
			{ "7" , "10" }, // 6
			{ "10" , "4" } // 7
		};
	
		SimpleRelation originalRelation = new SimpleRelation("Original");
		originalRelation.addAttribute(new SimpleNumericAttribute("first"));
		originalRelation.addAttribute(new SimpleNumericAttribute("second"));
		MemoryDataSet mockDataSet = new MemoryDataSet(csv, originalRelation);
		EuclideanSimilarity simFunction = new EuclideanSimilarity();
		MatrixFiller matrixFiller = new MatrixFiller(true, mockDataSet, database,simFunction);
		matrixFiller.fillMatrix();
				
		CliqueConstructor cliqueConstructor = new CliqueConstructor(database.connect(), matrixFiller.getSimilaritiesTable(), matrixFiller.getAveragesTable());
		cliqueConstructor.buildCliques(matrixFiller.getAveragesTable().getTotalAverage());
		
		instance = cliqueConstructor.getPoleTable();
		instance.printTable();
		
		assertEquals(6, instance.getSize());
		assertEquals(3, instance.getNumberElementPole(1));
		assertEquals(2, instance.getNumberPolesFound());
		
		
	}
}
